#!/usr/bin/env python

# col-cast-reduction.py
# Created by David Marsden
#
# Based on Krikor-color-cast-reduction
# Created by Tin Tran https://gimplearn.net#
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html#
#
# ------------
#| Change Log |
# ------------
# Version 1: 26/07/2022
# Version 2: 27/07/2022 Copy orig layer into new RGB layer and delete original
#						to eliminate any alpha channel.
# Version 3: 05/08/2022 Added a mask to correction colour layer to prevent contamination
#						of black by correction colour.
# Version 4: 10/09/2022 Changed to decompose/recompose to eliminate alpha channel.
#						Alpha channel re-applied after colour cast removal.
#						Any pre-existing selection is preserved and added at the end.
#						Layer name changed back to original at the end.
# Version 5: 23/06/2023 Option to stretch contrast of original image (default "Yes").
#						Appears to improve colour correction even for images of wide contrast.
#						Corrected error: wrong layer used to create Contrast layer.
#						Contrast layer opacity now defaults to 10%.
# Version 6: 25/06/2023 Found to be incredibly slow to pixelize large images. Added a layer which is scaled
#						to a very small size then pixelized. the colour is sampled, made the foreground colour
#						and then used to fill the "Colour" layer. The scaled layer is removed.
# Version 7: 22/08/2023 Replaced decompose/recompose with pdb.gimp_layer_flatten(layer) to remove alpha channel.
#                   	Removed contrast layer - not necessary - contrast, brightness etc. can be adjusted on base layer.
#
from gimpfu import *

def python_col_cast_reduction(image, layer, stretch):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()	
	
	width   = image.width																						# needed for plug_in_pixelize2
	height  = image.height																						# needed for plug_in_pixelize2
	
	x = 0																										# flag for selection (0 none, 1 has selection)	
	
	if not pdb.gimp_selection_is_empty(image):																	# check if there is a selection. Note: double negative!
		old_select = pdb.gimp_selection_save(image)																# save pre-existing selection to channel
		x = 1																									# flag says selection exists	
	
	pdb.gimp_selection_none(image)																				# remove any pre-existing selection
	
	layer_copy1 = pdb.gimp_layer_new_from_drawable(layer, image)												# layer new from drawable copies alpha channel	
	pdb.gimp_image_insert_layer(image,layer_copy1,None,0)														# insert layer
	
	pdb.gimp_layer_flatten(layer_copy1)																			# removes alpha layer if present
	
	if stretch == "Yes":
	
		pdb.gimp_drawable_levels_stretch(layer_copy1)															# equivalent to Auto Levels
		pdb.gimp_drawable_brightness_contrast(layer_copy1, 0.0, -0.1)											# reduce contrast 10%
	
	new_width = 100																								# scale layer to very small size
	new_height = 100 * (float (height) / float (width))															# to pixelise in short time
	layer_small = pdb.gimp_layer_new_from_drawable(layer_copy1, image)											# new layer to derive average colour
	pdb.gimp_image_insert_layer(image, layer_small, None, 0)													# insert layer
	pdb.gimp_layer_scale(layer_small, new_width, new_height, FALSE)												# scale layer
	pdb.plug_in_pixelize2(image, layer_small, new_width, new_height)											# blur pixelize with image width and image height	
	pdb.gimp_drawable_invert(layer_small, FALSE) 																# colors invert - FALSE for Linear	
	
	fill_colour=pdb.gimp_image_pick_color(image, layer_small, float(50), float(new_height/2), FALSE, FALSE, 0)	# sample colour from middle of layer_small
	pdb.gimp_context_set_foreground(fill_colour)																# set as foreground colour	
	
	pdb.gimp_image_remove_layer(image, layer_small)																# remove layer_small
	
	layer_copy2 = pdb.gimp_layer_new_from_drawable(layer_copy1, image)											# new layer for correction colour
	pdb.gimp_image_insert_layer(image, layer_copy2, None, 0)													# insert layer
	pdb.gimp_drawable_fill(layer_copy2, FILL_FOREGROUND)														# fill with inverted average colour
	layerName = "Colour"
	pdb.gimp_item_set_name(layer_copy2, layerName)																# name layer	
	pdb.gimp_layer_set_mode(layer_copy2, LAYER_MODE_OVERLAY)													# layer mode to overlay
		
	splinePoints = [[0.00,0.0, 0.05,1.0, 1.00,1.0]]																# create a curve to mask black parts of image
				
	for spline in splinePoints:	
				
		layer_copy4 = pdb.gimp_layer_new_from_drawable(layer_copy1, image)										# make a layer for applying curve
		pdb.gimp_image_insert_layer(image, layer_copy4, None, 0)												# insert layer
		
		pdb.gimp_drawable_levels_stretch(layer_copy4)															# equivalent to Auto Levels (maximise the spread before creating masks)
		
		pdb.gimp_drawable_curves_spline(layer_copy4, HISTOGRAM_VALUE, len(spline), spline)						# cycle through splinePoints list
		
	pdb.gimp_selection_all(image)																				# select image		
	pdb.gimp_edit_copy(layer_copy4)																				# copy to buffer	
		
	pdb.gimp_image_remove_layer(image, layer_copy4)																# remove splined layer											
 	pdb.gimp_selection_none(image)																				# select none
	
	mask = pdb.gimp_layer_create_mask(layer_copy2, ADD_MASK_WHITE)												# add mask to colour layer
	pdb.gimp_layer_add_mask(layer_copy2, mask)
	
	pdb.gimp_layer_set_edit_mask(layer_copy2, FALSE) 															# make mask active
	
	floating_sel=pdb.gimp_edit_paste(mask, TRUE)																# paste into mask										
	pdb.gimp_floating_sel_anchor(floating_sel)																	# colour is not added to black
 	
 	name =pdb.gimp_drawable_get_name(layer)																		# get name of original layer
 	
 	pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, layer)												# select original image												
 	mask = pdb.gimp_layer_create_mask(layer_copy1, ADD_MASK_SELECTION)											# create mask
 	pdb.gimp_layer_add_mask(layer_copy1, mask)																	# add mask to layer_copy1
 	pdb.gimp_layer_remove_mask(layer_copy1, MASK_APPLY)															# remove mask and convert to alpha channel
 	pdb.gimp_selection_none(image)																				# remove selection
 	pdb.gimp_image_remove_layer(image, layer)																	# remove original layer - no longer required
 	
	name =pdb.gimp_drawable_set_name(layer_copy1, name)															# set name back to original
	
	if x == 1:																									# if selection existed at start
		ch = image.channels[0]																					# top channel if more than one
		pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, ch)												# re-instate selection
		pdb.gimp_image_remove_channel(image, ch)																# remove channel used to store selection
		
	pdb.gimp_image_set_active_layer(image, layer_copy1)															# make base layer active
	
		
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_col_cast_reduction",
	"Col Cast Reduction",
	"Col Cast Reduction",
	"David Marsden",
	"David Marsden",
	"July 2022",
	"<Image>/Python-Fu/Col Cast Reduction...",             														# Menu path
	"RGB*, INDEXED*", 
	[
		(PF_RADIO, "stretch", "Stretch Contrast ", "Yes",
			(
				("Yes", "Yes"),
				("No", 	"No")
			)
		)	
	],
	[],
	python_col_cast_reduction)

main()
